#include <stdio.h>
#include <string.h>
#include <unistd.h>
#include <stdarg.h>

#include "sockutil.h"

/* read a response from the server. If TIMEOUT (seconds) elapse, deliver 
   SIGALRM to the process. Otherwise returns the response code, or 0 for EOF.*/

static int read_response(FILE *fp, int timeout)
{
    char buf[BUFSIZ], * ptr;

    for(;;)
     {
	alarm(timeout);
	
	ptr = fgets(buf, 1023, fp);
	
	alarm(0);

	if(!ptr)
	    return 0;

	ptr += strlen(ptr);

	if(*--ptr == '\n')
	 if(*--ptr == '\r')
	   *ptr = '\0';

	if(strlen(buf) < 3 || strspn(buf, "0123456789") != 3)
	    puts("*** malformed response");
	else 
	if (buf[3] == ' ' || buf[3] == '\0')
	    return atoi(buf);
    }
}	    


/* format a command string and send it to the server. 
   maximum command length 1021 bytes */

int send_command(FILE *fp, const char *fmt, ...)
{
    char buf[BUFSIZ];

    va_list va;
    
    va_start(va, fmt);
    vsprintf(buf, fmt, va);
    va_end(va);
    
   if(NULL == strchr(buf, '\n'))
     strcat(buf, "\n"); 

    return fputs(buf, fp) != EOF;
}

int main(int argc, char *argv[])
{
    struct sockaddr_in addr;
    int sock, rc, tmout = 5 * 60;   /* 5 minutes should be long enough */
    FILE * rfp, * wfp;
    char hostname[BUFSIZ], buffer[BUFSIZ];

    if(!argv[1])
     {
	fprintf(stderr, "usage: %s ip [subject]\n", argv[0]);
 	exit(2);
     }

    /* grok the server name */

    memset(&addr, 0, sizeof addr);

    addr.sin_family = AF_INET;
    addr.sin_port = htons(25);
    addr.sin_addr.s_addr = inet_addr(argv[1]);

    /* make the socket */

    sock = socket(AF_INET, SOCK_STREAM, IPPROTO_TCP);
    
    if(sock < 0)
     {
	perror("socket");
	exit(1);
     }

    /* contact server */

    rc = connect(sock, &addr, sizeof(addr));
    
    if(rc < 0)
     {
	perror("connect");
	exit(1);
     } 

    /* create read and write streams */

    rfp = sock_to_file(sock, "rt");
    wfp = sock_to_file(dup(sock), "wt");

    /* first response; expect 220 from server */

    if(read_response(rfp, tmout) != 220)
     {
	fprintf(stderr,"Unexpected response from server.\n");
	exit(1);
     }

    /* who are we? */

    gethostname(hostname, sizeof hostname);

    /* be polite */

    send_command(wfp, "HELO %s", hostname);
    read_response(rfp, tmout);

    send_command(wfp, "MAIL FROM: cluster-cloner@localhost");
    read_response(rfp, tmout);
    send_command(wfp, "RCPT TO: root");
    read_response(rfp, tmout);
    send_command(wfp, "DATA");
    send_command(wfp, "Subject: %s\n", argv[2] ? argv[2] : "No subject");
    read_response(rfp, tmout);

    while(fgets(buffer, sizeof buffer, stdin))
       send_command(wfp, buffer);
     
   send_command(wfp, ".");
   read_response(rfp, tmout);

    /* all done */

    send_command(wfp, "QUIT");
    read_response(rfp, tmout);

    /* This one should get EOF */

    read_response(rfp, tmout);

    /* close the streams and exit */

    fclose(rfp);
    fclose(wfp);

    return 0;
}
